@@ -24,7 +24,9 @@ import threading
 import time
 from abc import abstractmethod
 from collections import defaultdict
-from typing import TYPE_CHECKING, Any, Callable, Final
+from typing import TYPE_CHECKING, Any, Callable, Final, Generic, TypeVar, cast, overload
+
+from typing_extensions import ParamSpec
 
 from streamlit import type_util
 from streamlit.dataframe_util import is_unevaluated_data_object
@@ -51,8 +53,6 @@ from streamlit.runtime.scriptrunner_utils.script_run_context import (
 )
 
 if TYPE_CHECKING:
-    from types import FunctionType
-
     from streamlit.runtime.caching.cache_type import CacheType
 
 _LOGGER: Final = get_logger(__name__)
@@ -61,8 +61,12 @@ _LOGGER: Final = get_logger(__name__)
 # is exposed here as a constant so that it can be patched in unit tests.
 TTLCACHE_TIMER = time.monotonic
 
+# Type-annotate the cached function.
+P = ParamSpec("P")
+R = TypeVar("R")
 
-class Cache:
+
+class Cache(Generic[R]):
     """Function cache interface. Caches persist across script runs."""
 
     def __init__(self) -> None:
@@ -70,7 +74,7 @@ class Cache:
         self._value_locks_lock = threading.Lock()
 
     @abstractmethod
-    def read_result(self, value_key: str) -> CachedResult:
+    def read_result(self, value_key: str) -> CachedResult[R]:
         """Read a value and associated messages from the cache.
 
         Raises
@@ -82,7 +86,7 @@ class Cache:
         raise NotImplementedError
 
     @abstractmethod
-    def write_result(self, value_key: str, value: Any, messages: list[MsgData]) -> None:
+    def write_result(self, value_key: str, value: R, messages: list[MsgData]) -> None:
         """Write a value and associated messages to the cache, overwriting any existing
         result that uses the value_key.
         """
@@ -118,7 +122,7 @@ class Cache:
         raise NotImplementedError
 
 
-class CachedFuncInfo:
+class CachedFuncInfo(Generic[P, R]):
     """Encapsulates data for a cached function instance.
 
     CachedFuncInfo instances are scoped to a single script run - they're not
@@ -127,13 +131,15 @@ class CachedFuncInfo:
 
     def __init__(
         self,
-        func: FunctionType,
-        show_spinner: bool | str,
+        func: Callable[P, R],
         hash_funcs: HashFuncsDict | None,
+        show_spinner: bool | str,
+        show_time: bool = False,
     ) -> None:
         self.func = func
-        self.show_spinner = show_spinner
         self.hash_funcs = hash_funcs
+        self.show_spinner = show_spinner
+        self.show_time = show_time
 
     @property
     def cache_type(self) -> CacheType:
@@ -143,12 +149,12 @@ class CachedFuncInfo:
     def cached_message_replay_ctx(self) -> CachedMessageReplayContext:
         raise NotImplementedError
 
-    def get_function_cache(self, function_key: str) -> Cache:
+    def get_function_cache(self, function_key: str) -> Cache[R]:
         """Get or create the function cache for the given key."""
         raise NotImplementedError
 
 
-def make_cached_func_wrapper(info: CachedFuncInfo) -> Callable[..., Any]:
+def make_cached_func_wrapper(info: CachedFuncInfo[P, R]) -> CachedFunc[P, R]:
     """Create a callable wrapper around a CachedFunctionInfo.
 
     Calling the wrapper will return the cached value if it's already been
@@ -159,19 +165,19 @@ def make_cached_func_wrapper(info: CachedFuncInfo) -> Callable[..., Any]:
     some or all of the wrapper's cached values.
     """
     cached_func = CachedFunc(info)
-    return functools.update_wrapper(cached_func, info.func)
+    return cast("CachedFunc[P, R]", functools.update_wrapper(cached_func, info.func))
 
 
-class BoundCachedFunc:
+class BoundCachedFunc(Generic[P, R]):
     """A wrapper around a CachedFunc that binds it to a specific instance in case of
     decorated function is a class method.
     """
 
-    def __init__(self, cached_func: CachedFunc, instance: Any) -> None:
+    def __init__(self, cached_func: CachedFunc[P, R], instance: Any) -> None:
         self._cached_func = cached_func
         self._instance = instance
 
-    def __call__(self, *args: Any, **kwargs: Any) -> Any:
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
         return self._cached_func(self._instance, *args, **kwargs)
 
     def __repr__(self) -> str:
@@ -188,8 +194,8 @@ class BoundCachedFunc:
             self._cached_func.clear()
 
 
-class CachedFunc:
-    def __init__(self, info: CachedFuncInfo) -> None:
+class CachedFunc(Generic[P, R]):
+    def __init__(self, info: CachedFuncInfo[P, R]) -> None:
         self._info = info
         self._function_key = _make_function_key(info.cache_type, info.func)
 
@@ -203,7 +209,7 @@ class CachedFunc:
 
         return functools.update_wrapper(BoundCachedFunc(self, instance), self)
 
-    def __call__(self, *args: Any, **kwargs: Any) -> Any:
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
         """The wrapper. We'll only call our underlying function on a cache miss."""
 
         spinner_message: str | None = None
@@ -223,7 +229,7 @@ class CachedFunc:
         func_args: tuple[Any, ...],
         func_kwargs: dict[str, Any],
         spinner_message: str | None = None,
-    ) -> Any:
+    ) -> R:
         # Retrieve the function's cache object. We must do this "just-in-time"
         # (as opposed to in the constructor), because caches can be invalidated
         # at any time.
@@ -253,14 +259,14 @@ class CachedFunc:
         # on behalf of the user.
         is_nested_cache_function = in_cached_function.get()
         spinner_or_no_context = (
-            spinner(spinner_message, _cache=True)
+            spinner(spinner_message, _cache=True, show_time=self._info.show_time)
             if spinner_message is not None and not is_nested_cache_function
             else contextlib.nullcontext()
         )
         with spinner_or_no_context:
             return self._handle_cache_miss(cache, value_key, func_args, func_kwargs)
 
-    def _handle_cache_hit(self, result: CachedResult) -> Any:
+    def _handle_cache_hit(self, result: CachedResult[R]) -> R:
         """Handle a cache hit: replay the result's cached messages, and return its
         value.
         """
@@ -273,11 +279,11 @@ class CachedFunc:
 
     def _handle_cache_miss(
         self,
-        cache: Cache,
+        cache: Cache[R],
         value_key: str,
         func_args: tuple[Any, ...],
         func_kwargs: dict[str, Any],
-    ) -> Any:
+    ) -> R:
         """Handle a cache miss: compute a new cached value, write it back to the cache,
         and return that newly-computed value.
         """
@@ -346,6 +352,15 @@ class CachedFunc:
                     return_value=computed_value, func=self._info.func
                 )
 
+    @overload
+    def clear(self) -> None: ...
+
+    @overload
+    def clear(self, *args: P.args, **kwargs: P.kwargs) -> None: ...
+
+    @overload
+    def clear(self, *args: Any, **kwargs: Any) -> None: ...
+
     def clear(self, *args: Any, **kwargs: Any) -> None:
         """Clear the cached function's associated cache.
 
@@ -398,7 +413,7 @@ class CachedFunc:
 
 def _make_value_key(
     cache_type: CacheType,
-    func: FunctionType,
+    func: Callable[..., Any],
     func_args: tuple[Any, ...],
     func_kwargs: dict[str, Any],
     hash_funcs: HashFuncsDict | None,
@@ -463,7 +478,7 @@ def _make_value_key(
     return value_key
 
 
-def _make_function_key(cache_type: CacheType, func: FunctionType) -> str:
+def _make_function_key(cache_type: CacheType, func: Callable[..., Any]) -> str:
     """Create the unique key for a function's cache.
 
     A function's key is stable across reruns of the app, and changes when
@@ -503,7 +518,7 @@ def _make_function_key(cache_type: CacheType, func: FunctionType) -> str:
     return func_hasher.hexdigest()
 
 
-def _get_positional_arg_name(func: FunctionType, arg_index: int) -> str | None:
+def _get_positional_arg_name(func: Callable[..., Any], arg_index: int) -> str | None:
     """Return the name of a function's positional argument.
 
     If arg_index is out of range, or refers to a parameter that is not a
